@@ -9,6 +9,16 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
+"""Fixtures for writing tests for code using oslo.versionedobjects
+
+.. note::
+
+   This module has several extra dependencies not needed at runtime
+   for production code, and therefore not installed by default. To
+   ensure those dependencies are present for your tests, add
+   ``oslo.versionedobjects[fixtures]`` to your list of test dependencies.
+
+"""
 
 from collections import OrderedDict
 import hashlib
@@ -82,6 +92,23 @@ class FakeIndirectionAPI(base.VersionedObjectIndirectionAPI):
             context=context)
             if isinstance(result, base.VersionedObject) else result)
 
+    def object_class_action_versions(self, context, objname, objmethod,
+                                     object_versions, args, kwargs):
+        objname = six.text_type(objname)
+        objmethod = six.text_type(objmethod)
+        object_versions = {six.text_type(o): six.text_type(v)
+                           for o, v in object_versions.items()}
+        args, kwargs = self._canonicalize_args(context, args, kwargs)
+        objver = object_versions[objname]
+        cls = base.VersionedObject.obj_class_from_name(objname, objver)
+        with mock.patch('oslo_versionedobjects.base.VersionedObject.'
+                        'indirection_api', new=None):
+            result = getattr(cls, objmethod)(context, *args, **kwargs)
+        return (base.VersionedObject.obj_from_primitive(
+            result.obj_to_primitive(target_version=objver),
+            context=context)
+            if isinstance(result, base.VersionedObject) else result)
+
     def object_backport(self, context, objinst, target_version):
         raise Exception('not supported')
 
